//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <KStrings.hpp>
#include <KUtils.hpp>
#include "Main.h"
#include "Global.h"
#include "selectdb.h"
#include "querprms.h"
#include "about.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ddlabel"
#pragma link "kbdeinfo"
#pragma link "kdbstruc"
#pragma link "kqrybase"
#pragma link "kqrybase"
#pragma link "kqfbase"
#pragma link "kquerpnl"
#pragma link "kqrycomb"
#pragma link "ACRMain"
#pragma link "ACRsq_dbinfo"
#pragma resource "*.dfm"


TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
  ACRDatabase1->Open();
  AppDir = FixPath(GetCurrentDir());
  ReadOptions();
  OpenQueryBase();
  QPanel->Activate();
  EnableControls();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormShow(TObject *Sender)
{
  LoadFormPosReg(this, "Software\\Korzh\\KQB");
  if (pnlMain->Visible)
    QPanel->SetFocus();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
  CheckIsQuerySaved();
  SaveFormPosReg(this, RegBaseKey);
  SaveOptions();
}

void __fastcall TMainForm::CheckIsQuerySaved(void)
{
  if (QPanel->Modified){
    if (Ask("Current query was modified. Save?") == IDYES)
      QBase->CurrentQuery->Save();
  }
}


//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenQueryBase(void)
{
  QBase->Close();
  qryResult->Close();
  memSQL->Clear();
  QPanel->Active = false;
  if (FileExists(ConcPath(WorkDir, "struct.dbs")))
    DBStructure->LoadFromFile(ConcPath(WorkDir, "struct.dbs"));
  QBase->WorkDir = WorkDir;
  QBase->Open();
}

void __fastcall TMainForm::EnableControls(void)
{
  bool E = QBase->Count > 0;
  pnlMain->Visible = E;
  sbSaveQuery->Enabled = E && QPanel->Modified;
  sbQueryParams->Enabled = E;
  sbDeleteQuery->Enabled = E;
  sbBuildQuery->Enabled = E;
  sbActivateQueryPanel->Enabled = E;
  sbShowSQL->Enabled = E;

  miSaveQuery->Enabled = E && QPanel->Modified;
  miQueryParams->Enabled = E;
  miDeleteQuery->Enabled = E;
  miBuildQuery->Enabled = E;
  miBuildQuery->Enabled = E;
  miResultFields->Enabled = E && qryResult->Active;

  sbText->Enabled  = E && qryResult->Active;
  sbHTML->Enabled  = E && qryResult->Active;
  sbDBASE->Enabled = E && qryResult->Active;
  sbPrint->Enabled = E && qryResult->Active;
  sbPreview->Enabled = E && qryResult->Active;

  miActivateQueryPanel->Enabled = E;
  miShowSQL->Enabled = E;
  miShowDescription->Enabled = E;
  miShowDescription->Checked = true;
}


void __fastcall TMainForm::miNewDBClick(TObject *Sender)
{
  dlgSelectDB = new TdlgSelectDB(NULL);
  try{
    dlgSelectDB->dlbWorkDir->Directory = AppDir;
    BDEInfo->GetDatabaseNames(dlgSelectDB->cbDatabase->Items);
    if (dlgSelectDB->CreateDB() == mrOk) {
      WorkDir = ConcPath(dlgSelectDB->dlbWorkDir->Directory, dlgSelectDB->edNewDir->Text);
      ForceDirectories(WorkDir);
      QPanel->ClearAll();
      ddlRFields->Caption = QPanel->QResult->RFields->Caption;
//      BDEInfo->DatabaseName = dlgSelectDB->cbDatabase->Text;
      DBStructure->ClearStructure();
      EditDBStructure(this);
      OpenQueryBase();
    }
  }
  __finally{
    delete dlgSelectDB;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miSelectDBClick(TObject *Sender)
{
  dlgSelectDB = new TdlgSelectDB(NULL);
  try{
    if (dlgSelectDB->SelectDB() == mrOk){
      WorkDir = dlgSelectDB->dlbWorkDir->Directory;
      OpenQueryBase();
    }
  }
  __finally{
    delete dlgSelectDB;
  }
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::EditDBStructure(TObject *Sender)
{
  // you have to save current query before structure changing
  // and then load it to prevent links loss to TDSTable, TDSField
  // and TDSOperat objects stored in TKDBStructure

  TStrings *Strings = new TStringList();
  try{
    QPanel->SaveToStrings(Strings);
    if (DBStructure->EditStructure()) {
      DBStructure->SaveToFile(ConcPath(WorkDir, "struct.dbs"));
      try{
        QPanel->LoadFromStrings(Strings);
      }
      catch (...){
        //structure was changed - need to clear the structure
        QPanel->ClearAll();
      }
      if (DBStructure->DefaultMainTable != NULL)
        QPanel->MainTableName = DBStructure->DefaultMainTable->TableName;
    }
  }
  __finally{
    delete Strings;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miExitClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::miResultFieldsClick(TObject *Sender)
{
  if (QPanel->EditQResult())
    ddlRFields->Caption = QPanel->QResult->RFields->Caption;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ddlRFieldsCustomDropDown(TObject *Sender,
      AnsiString &Value, bool &Accept)
{
  TRFDOptions Options;
  Options << rfdShowSorting;
  Accept = QPanel->ShowResultFieldsDlg(0, Options);
  if (Accept) 
    Value = QPanel->QResult->RFields->Caption;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miNewQueryClick(TObject *Sender)
{
  TdlgQueryParams *dlgQueryParams = new TdlgQueryParams(NULL);
  try{
    if (dlgQueryParams->ShowModal() == mrOk) {
      TKQueryFileItem *NewItem = (TKQueryFileItem *)(QBase->NewQuery());
      NewItem->FileName = dlgQueryParams->edFileName->Text;
      NewItem->QueryName = dlgQueryParams->edQueryName->Text;
      NewItem->Description = dlgQueryParams->memDescription->Lines->Text;
      QBase->AddItem(NewItem);
      dlgQueryParams->SaveSQLOptions(QPanel);
  //      NewItem.Save;
    }
  }
  __finally{
    delete dlgQueryParams;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditQueryParams(TObject *Sender)
{
  TKQueryFileItem *QI = (TKQueryFileItem *)(QBase->Queries[QBase->ItemIndex]);
  TdlgQueryParams *dlgQueryParams = new TdlgQueryParams(NULL);
  try {
    dlgQueryParams->edQueryName->Text = QI->QueryName;
    dlgQueryParams->edFileName->Text = QI->FileName;
    dlgQueryParams->memDescription->Lines->Text = QI->Description;
    dlgQueryParams->LoadSQLOptions(QPanel);
    if (dlgQueryParams->ShowModal() == mrOk){
      QI->QueryName = dlgQueryParams->edQueryName->Text;
      QI->FileName = dlgQueryParams->edFileName->Text;
      QI->Description = dlgQueryParams->memDescription->Lines->Text;
      dlgQueryParams->SaveSQLOptions(QPanel);
      QI->Changed();
    }
  }
  __finally{  
    delete dlgQueryParams;
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveCurrentQuery(TObject *Sender)
{
  QBase->CurrentQuery->Save();
  EnableControls();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DeleteCurrentQuery(TObject *Sender)
{
  if (Ask("Delete Query?") == IDYES)
    QBase->EraseQuery(QBase->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BuildQuery(TObject *Sender)
{
  Screen->Cursor = crHourGlass;
  try {
    QPanel->QResult->RFields->RegenerateAliases();
    QPanel->BuildQuery();
    memSQL->Lines->Assign(QPanel->QResult->SQL);
    if (qryResult->Active)
      qryResult->Close();
    qryResult->SQL->Assign(QPanel->QResult->SQL);
//    qryResult->DatabaseName = BDEInfo->DatabaseName;
    qryResult->Open();
    int I;
    for (I = 0;I < QPanel->QResult->RFields->Count; I++)
      (*qryResult->Fields)[I]->DisplayLabel = (*QPanel->QResult->RFields)[I]->Caption;
  }
  __finally {
    Screen->Cursor = crDefault;
    EnableControls();
  }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::QPanelActive(TObject *Sender)
{
  sbActivateQueryPanel->Down = QPanel->Active;
  miActivateQueryPanel->Checked = QPanel->Active;
  ddlRFields->Caption = QPanel->QResult->RFields->Caption;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::sbActivateQueryPanelClick(TObject *Sender)
{
  QPanel->Active = ! QPanel->Active;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::sbShowSQLClick(TObject *Sender)
{
  miShowSQL->Checked = ! miShowSQL->Checked;
  pcSQL->Visible = miShowSQL->Checked;
  sbShowSQL->Down = miShowSQL->Checked;
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::QPanelChange(TObject *Sender)
{
  EnableControls();
  lblDescription->Caption = QPanel->Description->Text;
  ddlRFields->Caption = QPanel->QResult->RFields->Caption;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::QPanelCustomValue(TFilterCond *Condition,
      int ValueIndex, AnsiString &AValue)
{
/*
  if ((Condition->Field->FieldType == ftDate || Condition->Field->FieldType == ftDateTime)
    && (Condition->Field->FieldName <> 'ShipDate')) {
    DateDialog = TDateDialog->Create(NULL);
    if AValue = '' then
      DateDialog->Date = Date
    else
      try
        DateDialog->Date = StrToDate(AValue);
      except
        DateDialog->Date = Date;
      end;
    DateDialog->ShowModal;

    if DateDialog->ModalResult = mrOK then
      AValue = DateToStr(DateDialog->Date);
    DateDialog->Free;
  }
  else
    ShowMessage("Custom editor\n" + Condition->CustomData->Text);
*/    
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::miAboutClick(TObject *Sender)
{
  TAboutBox *AboutBox = new TAboutBox(NULL);
  AboutBox->ShowModal();
  delete AboutBox;        
}
//---------------------------------------------------------------------------

