{*************************************************************}
{                      Simple Query pack                      }
{             Copyright  1998-2005 Korzh company             }
{               http://www.korzh.com/delphi/sq                }
{                   mailto:info@korzh.com                     }
{-------------------------------------------------------------}

////////////////////////////////////////////////////////////////////////////////
//
//                TACRsq_dbinfo component for
//                Accuracer Database System
//                Release Date: 08/05/2005
//                 AidAim Software, 2000-2005
//                http://www.aidaim.com
//                http://www.accuracer.com
//
////////////////////////////////////////////////////////////////////////////////


unit ACRsq_dbinfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, KDbInfo,
  ACRMain;

type
  TACRsq_dbinfo = class(TKDbInfo)
  private
    FACRDatabase: TACRDatabase;
  protected
 protected
    procedure AvailableDatabases(DBList : TStrings);override;
    procedure TablesByDatabase(TablesList : TStrings);override;
    procedure FieldsByTable(const TableName : string; FieldsList : TDBFieldList);override;
    procedure Notification( AComponent : TComponent;Operation : TOperation);override;
    function GetConnected: boolean; override;
  public
    constructor Create(AOwner : TComponent);override;
    procedure GetSQLValues(SQL : String;AValues,AItems : TStrings);override;
  published
    property AccuracerDatabase: TACRDatabase read FACRDatabase write FACRDatabase;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SimpleQuery', [TACRsq_dbinfo]);
end;



////////////////////////////////////////////////////////////////////////////////
//
// TACRsq_dbinfo
//
////////////////////////////////////////////////////////////////////////////////


//------------------------------------------------------------------------------
// get databae names
//------------------------------------------------------------------------------
procedure TACRsq_dbinfo.AvailableDatabases(DBList : TStrings);
begin
  if (FACRDatabase <> nil) then
    DBList.Add(FACRDatabase.DatabaseName);
end; // GetDatabaseNames


//------------------------------------------------------------------------------
// get table names from database
//------------------------------------------------------------------------------
procedure TACRsq_dbinfo.TablesByDatabase(TablesList: TStrings);
begin
  if (FACRDatabase <> nil) then
    FACRDatabase.GetTablesList(TablesList);
end; // TablesByDatabase


//------------------------------------------------------------------------------
// get fields list of the specified table
//------------------------------------------------------------------------------
procedure TACRsq_dbinfo.FieldsByTable(const TableName: string;
  FieldsList: TDBFieldList);
var
  table:      TACRtable;
  i:          Integer;
  FieldInfo:  TDBFieldInfo;
begin
  if (FACRDatabase = nil) then
    raise EKDBInfoError.Create('Database is not set in FieldsByTable');
  table := TACRTable.Create(nil);
  try
    table.DatabaseName := FACRDatabase.DatabaseName;
    table.TableName := TableName;

    table.FieldDefs.Update;
    FieldsList.Clear;
    for i := 0 to table.FieldDefs.Count - 1 do
     begin
      FieldInfo := TDBFieldInfo.Create;
      FieldInfo.FieldName := table.FieldDefs[i].Name;
      FieldInfo.FieldType := table.FieldDefs[i].DataType;
      FieldInfo.FieldSize := table.FieldDefs[i].Size;
      FieldsList.Add(FieldInfo);
     end;
  finally
    table.Free;
  end;
end; // FieldsBytable


//------------------------------------------------------------------------------
// get notification for removing connection
//------------------------------------------------------------------------------
procedure TACRsq_dbinfo.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if ((Operation = opRemove) and (AComponent = FACRDatabase)) then
    FACRDatabase := nil;
end; // Notification


//------------------------------------------------------------------------------
// return true if connected
//------------------------------------------------------------------------------
function TACRsq_dbinfo.GetConnected: boolean;
begin
  Result := False;
  if (FACRDatabase <> nil) then
   if (FACRDatabase.Connected) then
    Result := True;
end; // GetConnected


//------------------------------------------------------------------------------
// Create
//------------------------------------------------------------------------------
constructor TACRsq_dbinfo.Create(AOwner: TComponent);
begin
  inherited;
  FACRDatabase := nil;
end; // Create


//------------------------------------------------------------------------------
// get SQL values
//------------------------------------------------------------------------------
procedure TACRsq_dbinfo.GetSQLValues(SQL: String; AValues, AItems: TStrings);
var
  query : TACRQuery;
begin
  AValues.Clear;
  AItems.Clear;
  if (FACRDatabase = nil) then
    raise EKDBInfoError.Create('Database is not set in GetSQLValues');
  query := TACRQuery.Create(nil);
  try
    query.DatabaseName := FACRDatabase.DatabaseName;
    query.SQL.Text := SQL;
    try
      query.Open;
      if query.FieldCount < 2 then
        raise EKDBInfoError.Create('Field count must be >= 2');
      while not query.EOF do
      begin
        AValues.Add(query.Fields[0].AsString);
        AItems.Add(query.Fields[1].AsString);
        query.Next;
      end;
    except
      ;
    end;
  finally
    query.Free;
  end;
end; // GetSQLValues


end.
